
ins, del {
   padding-left: 2px !important;
   padding-right: 2px !important;
   color: #FFF !important;
   text-shadow: 0 -1px rgba(0, 0, 0, 0.3);
   border: 1px solid rgba(0, 0, 0, 0.1);
   border-radius: 3px 3px 3px 3px;
}

ins {
   background-color: #7FAE00 !important;
}

del {
   background-color: #C00 !important;
}
